
/*
  Remote VPLUS invocation facility
  Version 1.0 / 04.apr.02 - 12.apr.02
  Copyright (C) 2002  Lars Appel

  This package is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This package is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this package; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <stdlib.h>


char *buff = NULL;

/* 4 byte aligned buf is [totlen] [reclen] [data...] [reclen] [data...] ... */

int *base;
int *next;

/* base points to [totlen] field, next points to next avail [reclen] field */


/**************************************************************************
 !! base and next are int pointers => 4 byte steps in pointer arithmetic !!
 **************************************************************************/

void buf_init() {
  buff = malloc(BUF_SZ);
  if (buff == NULL) DEBUG();
  base = (int *) buff;
  *base = 4;
  next = base + 1;
}


void buf_put1st(void *adr, int len) {
  base = (int *) buff;
  *base = 4;
  next = base + 1;
  buf_putnxt(adr, len);
}

void buf_putnxt(void *adr, int len) {
  if (*base + 4 + len >= BUF_SZ) DEBUG();
  *next = len;
  memcpy(next + 1, adr, len);
  *base += (4 + len + 3) & (~0x3);
  next += (4 + len + 3) / 4;
}


void *buf_ref1st() {
  base = (int *) buff;
  next = base + 1;
  if ((next - base) >= *base) DEBUG();
  return next + 1;
}

void *buf_refnxt() {
  next += (4 + *next + 3) / 4;
  if ((next - base) >= *base) DEBUG();
  return next + 1;
}


void buf_get1st(void *adr) {
  base = (int *) buff;
  next = base + 1;
  if ((next - base) >= *base) DEBUG();
  memcpy(adr, next + 1, *next);
}


void buf_getnxt(void *adr) {
  next += (4 + *next + 3) / 4;
  if ((next - base) >= *base) DEBUG();
  memcpy(adr, next + 1, *next);
}


static int sock = 0;
static int serv = 0;


void buf_send() {
  int have, part;
  have = 0;
  while (have < *base) {
    part = send(sock, (buff + have), *base - have, 0);
    if (part == -1) DEBUG();
    have += part;
  }
}


void buf_recv() {
  int have, part;
  base = (int *) buff;
  have = 0;
  while (have < 4) {
    part = recv(sock, (buff + have), 4 - have, 0);
    if (part == -1) DEBUG();
    have += part;
  }
  while (have < *base) {
    part = recv(sock, (buff + have), *base - have, 0);
    if (part == -1) DEBUG();
    have += part;
  }
}


void srv_init()
{
  char *serv_port;
  struct sockaddr_in sa;
  int ret;
  int adrlen;

  if (buff == NULL) buf_init();

  serv = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (serv == -1) { perror("socket"); DEBUG(); }

  memset(&sa, 0, sizeof(sa));

  serv_port = getenv("SERV_PORT");
  if (serv_port == NULL) DEBUG();
  
  sa.sin_family = AF_INET;
  sa.sin_addr.s_addr = INADDR_ANY;
  sa.sin_port = htons(atoi(serv_port));

  ret = bind (serv, &sa, sizeof(sa));
  if (ret < 0) { perror("bind"); DEBUG(); }

  ret = listen(serv, 10);
  if (ret < 0) { perror("listen"); DEBUG(); }
}

void srv_wait() {
  struct sockaddr_in sa;
  int adrlen;

  memset(&sa, 0, sizeof(sa));
  adrlen = sizeof(sa);

  sock = accept (serv, &sa, &adrlen);
  if (sock == -1) { perror("accept"); DEBUG(); }
}

pid_t srv_fork() {
  pid_t child;
  child = fork();
  if (child == -1) {
    perror("fork"); DEBUG();
  }
  else if (child == 0) {
    close(serv); serv = 0;  // ??
  }
  else {
    close(sock); sock = 0;  // ??
  }
  return child;
}

void srv_exit()
{
  if (sock != 0) {
    if (shutdown(sock, 2) == -1) { perror("shutdown sock"); DEBUG(); }
  }
  if (serv != 0) {
    if (shutdown(serv, 2) == -1) { perror("shutdown serv"); DEBUG(); }
  }
}


void cli_init()
{
  char *serv_addr;
  char *serv_port;
  struct sockaddr_in sa;
  int ret;

  if (buff == NULL) buf_init();

  if (sock != 0) return;
  
  sock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (sock == -1) { perror("socket"); DEBUG(); }

  memset(&sa, 0, sizeof(sa));

  serv_addr = getenv("SERV_ADDR");
  if (serv_addr == NULL) DEBUG();

  serv_port = getenv("SERV_PORT");
  if (serv_port == NULL) DEBUG();

  sa.sin_family = AF_INET;
  sa.sin_addr.s_addr = inet_addr(serv_addr);
  sa.sin_port = htons(atoi(serv_port));

  ret = connect (sock, &sa, sizeof(sa));
  if (ret != 0) { perror("connect"); DEBUG(); }
}

void cli_exit()
{
  int req = R_FINISH;
  buf_put1st(&req, 4);
  buf_send();

  if (sock != 0) {
    if (shutdown(sock, 2) == -1) { perror("shutdown"); DEBUG(); }
  }
}

